/*
Title     :  PushByAngle – 3dsmax’s MAXscript modifier  allows to “push” object vertices outward or inward along the average vertex normals on the value which is calculated from the angle of two chosen objects. PushByAngle extended version of 3dsmax’s Push modifier. Modifier support any kind of geometry objects.

Copyright :  2015 veda3d.com
Author    : Royal Ghost

Version   : 1.0.0
					 - Initial release for 3ds max 2012-2016+.
						
Usage	    : 			
						http://veda3d.com/pushbyangle-modifier/

			
Homepage  : http://veda3d.com
*/
--
plugin modifier PushByAngle
	name:"PushByAngle"
	extends:Push replaceUI:true 
	classID:#(0x57f7d0c3, 0x6f979aca)
	version:1
(
parameters main rollout:params
(
	cur_floatangle type:#float ui:cur_floatangle
	cur_angle type:#float ui:cur_angle
	multiplier type:#float ui:multiplier default:1.0
	relax_angle type:#float ui:relax_angle default:180.0
)
rollout params "Parameters"
(
pickbutton btn_pick1Obj "Pick Object 1" width:125
pickbutton btn_pick2Obj "Pick Object 2"	width:125

spinner cur_floatangle "Cur.Float Angle: " range:[-360,360,0] enabled:false
spinner multiplier "Multiplier: " range:[-100,100,1]
spinner relax_angle "Relaxed Angle: " range:[-360,360,0]
spinner cur_angle "Current Angle: " range:[-360,360,0] enabled:false
button btn_set_relax "Set Relax Angle" width:112 height:21
label lbl_about "by Royal Ghost" offset:[0,10]
hyperLink  v3dUrl "veda3d.com" align:#center address:"http://veda3d.com" 
	
	on params open do
	(
		if this.cur_angle.controller.VariableExists "obj1" == true then btn_pick1Obj.text = (this.cur_angle.controller.GetNode "obj1").name
		if this.cur_angle.controller.VariableExists "obj2" == true then btn_pick2Obj.text = (this.cur_angle.controller.GetNode "obj2").name
	)
	
	on btn_autowire pressed do
	(
	--format "%\n" selection[1].modifiers[#musclestrandmod].cur_floatangle
	paramWire.connect selection[1].modifiers[#musclestrandmod][#cur_floatangle] (selection[1].modifiers[#musclestrandmod][#Push][#Push_Value]) "cur_floatangle"
	)
	
	on btn_pick1Obj picked obj do
	(
		btn_pick1Obj.text = obj.name
		this.cur_angle.controller.addNode "obj1" obj
		if this.cur_angle.controller.VariableExists "obj1" == true and this.cur_angle.controller.VariableExists "obj2" == true then
		(
	this.cur_angle.controller.script = "try (V1 = obj1.transform.pos - obj.transform.pos
V2 = obj2.transform.pos - obj.transform.pos
N1 = normalize V1
N2 = normalize V2
Ang = acos (dot N1 N2))catch(0)"			
		)
		--print (this.cur_angle.controller.VariableExists "obj2")
	)
	on btn_pick2Obj picked obj do
	(
		btn_pick2Obj.text = obj.name
		this.cur_angle.controller.addNode "obj2" obj
		if this.cur_angle.controller.VariableExists "obj1" == true and this.cur_angle.controller.VariableExists "obj2" == true then
		(
	this.cur_angle.controller.script = "try (V1 = obj1.transform.pos - obj.transform.pos
V2 = obj2.transform.pos - obj.transform.pos
N1 = normalize V1
N2 = normalize V2
Ang = acos (dot N1 N2))catch(0)"					
		)
		--print (this.cur_angle.controller.VariableExists "obj2")
	)

	on relax_angle changed val do
	(
		if this.cur_angle.controller.VariableExists "obj1" == true and this.cur_angle.controller.VariableExists "obj2" == true then
		(
		obj = refs.dependentNodes this
		local cur_floatangle_string = "(("+(this.relax_angle as string)+"-cur_angle)*"+((1.0/this.relax_angle) as string)+")*" + (this.multiplier as string)
		paramWire.connect obj[1].modifiers[#PushByAngle][#cur_angle] obj[1].modifiers[#PushByAngle][#cur_floatangle] cur_floatangle_string
		)
	)
	on multiplier changed val do
	(
		if this.cur_angle.controller.VariableExists "obj1" == true and this.cur_angle.controller.VariableExists "obj2" == true then
		(
		obj = refs.dependentNodes this
		local cur_floatangle_string = "(("+(this.relax_angle as string)+"-cur_angle)*"+((1.0/this.relax_angle) as string)+")*" + (this.multiplier as string)
		paramWire.connect obj[1].modifiers[#PushByAngle][#cur_angle] obj[1].modifiers[#PushByAngle][#cur_floatangle] cur_floatangle_string
		)	
	)
	on btn_set_relax pressed do
	(
		if this.cur_angle.controller.VariableExists "obj1" == true and this.cur_angle.controller.VariableExists "obj2" == true then
		(
		obj = refs.dependentNodes this
		this.relax_angle = this.cur_angle
		local cur_floatangle_string = "(("+(this.relax_angle as string)+"-cur_angle)*"+((1.0/this.relax_angle) as string)+")*" + (this.multiplier as string)
		paramWire.connect obj[1].modifiers[#PushByAngle][#cur_angle] obj[1].modifiers[#PushByAngle][#cur_floatangle] cur_floatangle_string
		)
	)
	
) -- end of rollout
	
	on attachedToNode obj do
	(
	obj = refs.dependentNodes this
	this.cur_angle.controller = float_script()
	this.cur_angle.controller.addNode "obj" obj[1]

	local cur_floatangle_string = "(("+(this.relax_angle as string)+"-cur_angle)*"+((1.0/this.relax_angle) as string)+")*" + (this.multiplier as string)

--	print cur_floatangle_string
	paramWire.connect obj[1].modifiers[#PushByAngle][#cur_angle] obj[1].modifiers[#PushByAngle][#cur_floatangle] cur_floatangle_string
	
	paramWire.connect obj[1].modifiers[#PushByAngle][#cur_floatangle] (selection[1].modifiers[#PushByAngle][#Push][#Push_Value]) "cur_floatangle"
	)--end of attachedToNode
)--end of plugin